#include "libasset.h"
#include <stdio.h>
#include <errno.h>

int main( int argc, char *argv[] )
{
    ASSET_MANAGER	mgr;
    ASSET_HANDLE	res;
    char		*buf1, *buf2;
    FILE		*fh;
    long		size = 0;

    if (argc != 4)
    {
	fprintf( stderr, "Usage: compare <zippath> <asset_path> <file_path>\n" );
	exit( 1 );
    }

    mgr = new_asset_manager();
    if (mgr == NULL)
	printf( "Failed to allocate asset manager!\n" );
	
    if (set_asset_path( mgr, argv[ 1 ] ) != 0)
    {
	fprintf( stderr, "%s\n", get_asset_error( mgr, NULL ) );
	exit( 1 );
    }

    fh = fopen( argv[ 3 ], "rb" );
    if (fh == NULL)
    {
	fprintf( stderr, "%s\n", strerror( errno ) );
	exit( 1 );
    }
    printf( "Opened file handle OK\n" );
    
    res = open_asset( mgr, argv[ 2 ] );
    if (res == NULL)
    {
	fprintf( stderr, "%s\n", get_asset_error( mgr, NULL ) );
	exit( 1 );
    }
	
    printf( "Opened asset handle OK\n" );

    buf1 = alloca( 1000 );
    buf2 = alloca( 1000 );

    for (;;)
    {
	long	result1, result2;
	int	i;

	result1 = fread( buf1, 1, 1000, fh );
	if (result1 < 0)
	{
	    fprintf( stderr, "%s\n", strerror( errno ) );
	    exit( 1 );
	}
	    
//	printf( "Read %d bytes from file\n", result1 );

	result2 = read_asset( res, buf2, 1000 );
	if (result2 < 0)
	{
	    fprintf( stderr, "%s\n", get_asset_error( mgr, NULL ) );
	    exit( 1 );
	}

//	printf( "Read %d bytes from asset\n", result2 );

	if (result1 != result2)
	{
	    fprintf( stderr, "Asset size %d != File size %d\n",
		     size + result1, size + result2 );
	    exit( 1 );
	}

	for (i = 0; i < result1; i++)
	{
	    if (buf1[ i ] != buf2[ i ])
	    {
		fprintf( stderr, "Mismatch at byte %d\n", size + i );
		exit( 1 );
	    }
	}

	if (result1 < 1000) break;

	size += result1;
    }
    
    printf( "Files match!\n" );
    fclose( fh );
    close_asset( res );
    delete_asset_manager( mgr );
}
