#include "libasset.h"
#include <stdio.h>
#include <errno.h>

extern int libasset_debug;

int main( int argc, char *argv[] )
{
    ASSET_MANAGER	mgr;
    ASSET_HANDLE	res;
    char		*buf;
    FILE		*fh;
    long		size = 0;

    libasset_debug = 1;

    if (argc != 4)
    {
	fprintf( stderr, "Usage: extract <zippath> <asset_path> <file_path>\n" );
	exit( 1 );
    }

    mgr = new_asset_manager();
    if (mgr == NULL)
	printf( "Failed to allocate asset manager!\n" );
	
    if (set_asset_path( mgr, argv[ 1 ] ) != 0)
    {
	fprintf( stderr, "%s\n", get_asset_error( mgr, NULL ) );
	exit( 1 );
    }

    fh = fopen( argv[ 3 ], "wb" );
    if (fh == NULL)
    {
	fprintf( stderr, "%s\n", strerror( errno ) );
	exit( 1 );
    }
    
    res = open_asset( mgr, argv[ 2 ] );
    if (res == NULL)
    {
	fprintf( stderr, "%s\n", get_asset_error( mgr, NULL ) );
	exit( 1 );
    }
	
    buf = alloca( 8192 );

    for (;;)
    {
	long	result;

	result = read_asset( res, buf, 8192 );
	if (result < 0)
	{
	    fprintf( stderr, "%s\n", get_asset_error( mgr, NULL ) );
	    exit( 1 );
	}

	if (fwrite( buf, 1, result, fh ) != result)
	{
	    fprintf( stderr, "%s\n", strerror( errno ) );
	    exit( 1 );
	}
	    
	size += result;
	if (result < 1000) break;
    }
    
    printf( "File extracted, %d bytes\n", size );
    fclose( fh );
    close_asset( res );
    delete_asset_manager( mgr );
}
